import psycopg2
import psycopg2.extras # For dictionary cursor
import logging
from contextlib import contextmanager
from config import Config
from urllib.parse import urlparse

# Configure basic logging for this module
logger = logging.getLogger(__name__)

@contextmanager
def get_db_connection():
    """Provides a database connection context."""
    conn = None
    try:
        # Connect using the URI from Config
        conn = psycopg2.connect(Config.DATABASE_URI)
        # Yield the connection; cursor will be created within the 'with' block where needed
        yield conn
        conn.commit() # Commit changes if no exceptions occurred
    except (Exception, psycopg2.DatabaseError) as error:
        logger.error(f"Database connection error: {error}", exc_info=True)
        if conn:
            conn.rollback() # Rollback changes on error
        raise # Re-raise the exception after logging/rollback
    finally:
        if conn:
            conn.close()

def get_app_settings():
    """Fetches API keys and other application settings."""
    settings = {
        'openai': None, 'google': None, 'anthropic': None, 'google_cse_id': None,
        'deep_dive_enabled': False, 'deep_dive_search_depth': 25, # Defaults
        'pipeline_interval_minutes': 60 # Default interval in minutes
    }
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            # Select all relevant columns
            cursor.execute("""
                SELECT openai_api_key, google_api_key, anthropic_api_key, google_cse_id,
                       deep_dive_enabled, deep_dive_search_depth, pipeline_interval_minutes
                FROM app_settings WHERE id = 1
            """)
            row = cursor.fetchone()
            if row:
                settings['openai'] = row['openai_api_key']
                settings['google'] = row['google_api_key']
                settings['anthropic'] = row['anthropic_api_key']
                settings['google_cse_id'] = row['google_cse_id']
                settings['deep_dive_enabled'] = bool(row['deep_dive_enabled']) if row['deep_dive_enabled'] is not None else False
                settings['deep_dive_search_depth'] = row['deep_dive_search_depth'] if row['deep_dive_search_depth'] is not None else 25
                settings['pipeline_interval_minutes'] = row['pipeline_interval_minutes'] if row['pipeline_interval_minutes'] is not None else 60
            else:
                logger.warning("Application settings not found in app_settings table (id=1). Using defaults.")
    except Exception as e:
        logger.error(f"Failed to fetch application settings: {e}", exc_info=True)
    return settings

def get_system_prompt(agent_name):
    """Fetches the system prompt for a specific agent."""
    prompt = None
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT prompt FROM system_prompts WHERE agent_name = %s", (agent_name,))
            row = cursor.fetchone()
            if row:
                prompt = row['prompt']
            else:
                logger.warning(f"System prompt for agent '{agent_name}' not found.")
    except Exception as e:
        logger.error(f"Failed to fetch system prompt for {agent_name}: {e}", exc_info=True)
    return prompt

def get_domain(url: str) -> str | None:
    """Extracts the domain name (e.g., 'example.com') from a URL."""
    if not url:
        return None
    try:
        # Add scheme if missing for urlparse to work correctly
        if not url.startswith(('http://', 'https://')):
            url = 'http://' + url
        parsed_uri = urlparse(url)
        domain = parsed_uri.netloc
        # Remove 'www.' prefix if present
        if domain.startswith('www.'):
            domain = domain[4:]
        return domain.lower() # Return lowercase domain
    except Exception as e:
        logger.error(f"Failed to parse domain from URL '{url}': {e}")
        return None
